#include "udp.h"
//-------------------------------------------------------------
static const char *TAG = "udp";
//-------------------------------------------------------------
//-------------------------------------------------------------
typedef struct
{
  unsigned char y_pos;
  unsigned char x_pos;
  char *str;
} qLCDData;
//------------------------------------------------
xQueueHandle lcd_string_queue = NULL;
//------------------------------------------------
void vLCDTask(void* arg)
{
  BaseType_t xStatus;
  qLCDData xReceivedData;
  for(;;) {
    xStatus = xQueueReceive(lcd_string_queue, &xReceivedData, 10000 /portTICK_RATE_MS);
    if (xStatus == pdPASS)
    {
      LCD_SetPos(xReceivedData.x_pos,xReceivedData.y_pos);
      LCD_String(xReceivedData.str);
    }
  }
}
//------------------------------------------------
static void recv_task(void *pvParameters)
{
  char buf[10] = {};
  int *sock = (int*) pvParameters;
  qLCDData xLCDData;
  char str1[10];
  xLCDData.y_pos = 1;
  xLCDData.str = str1;
  for(short i=0;;i++)
  {
    recv(*sock, buf, sizeof(buf), 0);
    snprintf(str1, sizeof(str1), "%6d", *(short*)buf);
    xQueueSendToBack(lcd_string_queue, &xLCDData, 0);
  }
}
//------------------------------------------------
void udp_task(void *pvParameters)
{
  TaskHandle_t xLCDTaskHandle = NULL, xRecvTask = NULL;
  portTickType xLastWakeTime;
  int sockfd;
  struct sockaddr_in servaddr, cliaddr;
  lcd_string_queue = xQueueCreate(10, sizeof(qLCDData));
  xTaskCreate(vLCDTask, "vLCDTask", 2048, NULL, 2, &xLCDTaskHandle);
  ESP_LOGI(TAG, "Create socket...\n");
  if ( (sockfd = socket(AF_INET, SOCK_DGRAM, IPPROTO_IP)) < 0 ) {
    ESP_LOGE(TAG, "socket not created\n");
    vTaskDelete(NULL);
  }
  memset(&servaddr, 0, sizeof(servaddr));
  memset(&cliaddr, 0, sizeof(cliaddr));
  //   
  cliaddr.sin_family    = AF_INET; // IPv4
  cliaddr.sin_addr.s_addr = INADDR_ANY;
  cliaddr.sin_port = htons(CONFIG_CLIENT_PORT);
  //    
  if (bind(sockfd, (const struct sockaddr *)&cliaddr,  sizeof(struct sockaddr_in)) < 0 )
  {
    ESP_LOGE(TAG, "socket not binded\n");
    vTaskDelete(xLCDTaskHandle);
    vQueueDelete(lcd_string_queue);
    vTaskDelete(NULL);
  }
  ESP_LOGI(TAG, "socket was binded\n");
  //   
  servaddr.sin_family    = AF_INET; // IPv4
  servaddr.sin_addr.s_addr = inet_addr(CONFIG_SERVER_IP);
  servaddr.sin_port = htons(CONFIG_SERVER_PORT);
  xTaskCreate(recv_task, "recv_task", 4096, (void*)&sockfd, 5, &xRecvTask);
  xLastWakeTime = xTaskGetTickCount();
  for(short i=0;;i++)
  {
    sendto(sockfd, &i, 2,  0, (struct sockaddr*) &servaddr,  sizeof(servaddr));
    vTaskDelayUntil( &xLastWakeTime, ( 100 / portTICK_RATE_MS ) );
  }
  shutdown(sockfd, 0);
  close(sockfd);
  vTaskDelete(xRecvTask);
  vTaskDelete(xLCDTaskHandle);
  vQueueDelete(lcd_string_queue);
  vTaskDelete(NULL);
}
//-------------------------------------------------------------
